@@ -38,6 +38,7 @@ module Data.Conduit.Combinators
 
       -- ** I\/O
     , sourceFile
+    , sourceFileBS
     , sourceHandle
     , sourceIOHandle
     , stdin
@@ -89,10 +90,13 @@ module Data.Conduit.Combinators
     , sinkLazyBuilder
     , sinkNull
     , awaitNonNull
+    , head
+    , headDef
     , headE
     , peek
     , peekE
     , last
+    , lastDef
     , lastE
     , length
     , lengthE
@@ -120,6 +124,7 @@ module Data.Conduit.Combinators
 
       -- ** I\/O
     , sinkFile
+    , sinkFileBS
     , sinkHandle
     , sinkIOHandle
     , print
@@ -187,6 +192,7 @@ module Data.Conduit.Combinators
     , vectorBuilder
     , mapAccumS
     , peekForever
+    , peekForeverE
     ) where
 
 -- BEGIN IMPORTS
@@ -212,7 +218,7 @@ import qualified Data.Conduit.Filesystem as CF
 import           Data.Conduit.Internal       (ConduitM (..), Pipe (..))
 import qualified Data.Conduit.List           as CL
 import           Data.IOData
-import           Data.Maybe                  (isNothing, isJust)
+import           Data.Maybe                  (fromMaybe, isNothing, isJust)
 import           Data.Monoid                 (Monoid (..))
 import           Data.MonoTraversable
 import qualified Data.Sequences              as Seq
@@ -419,6 +425,14 @@ sourceFile :: (MonadResource m, IOData a, MonoFoldable a) => FilePath -> Produce
 sourceFile fp = sourceIOHandle (SIO.openFile fp SIO.ReadMode)
 {-# INLINE sourceFile #-}
 
+-- | 'sourceFile' specialized to 'ByteString' to help with type
+-- inference.
+--
+-- @since 1.0.7
+sourceFileBS :: MonadResource m => FilePath -> Producer m ByteString
+sourceFileBS = sourceFile
+{-# INLINE sourceFileBS #-}
+
 -- | Read all data from the given @Handle@.
 --
 -- Does not close the @Handle@ at any point.
@@ -1029,6 +1043,18 @@ awaitNonNull =
     go' = maybe go (return . Just) . NonNull.fromNullable
 {-# INLINE awaitNonNull #-}
 
+-- | Take a single value from the stream, if available.
+--
+-- Since 1.0.5
+head :: Monad m => Consumer a m (Maybe a)
+head = CL.head
+
+-- | Same as 'head', but returns a default value if none are available from the stream.
+--
+-- Since 1.0.5
+headDef :: Monad m => a -> Consumer a m a
+headDef a = fromMaybe a <$> head
+
 -- | Get the next element in the chunked stream.
 --
 -- Since 1.0.0
@@ -1080,6 +1106,12 @@ lastC =
     loop prev = await >>= maybe (return $ Just prev) loop
 STREAMING0(last, lastC, lastS)
 
+-- | Same as 'last', but returns a default value if none are available from the stream.
+--
+-- Since 1.0.5
+lastDef :: Monad m => a -> Consumer a m a
+lastDef a = fromMaybe a <$> last
+
 -- | Retrieve the last element in the chunked stream, if present.
 --
 -- Subject to fusion
@@ -1300,6 +1332,14 @@ sinkFile :: (MonadResource m, IOData a) => FilePath -> Consumer a m ()
 sinkFile fp = sinkIOHandle (SIO.openFile fp SIO.WriteMode)
 {-# INLINE sinkFile #-}
 
+-- | 'sinkFile' specialized to 'ByteString' to help with type
+-- inference.
+--
+-- @since 1.0.7
+sinkFileBS :: MonadResource m => FilePath -> Consumer ByteString m ()
+sinkFileBS = sinkFile
+{-# INLINE sinkFileBS #-}
+
 -- | Print all incoming values to stdout.
 --
 -- Subject to fusion
@@ -2130,3 +2170,24 @@ peekForever inner =
         case mx of
             Nothing -> return ()
             Just _ -> inner >> loop
+
+-- | Run a consuming conduit repeatedly, only stopping when there is no more
+-- data available from upstream.
+--
+-- In contrast to 'peekForever', this function will ignore empty
+-- chunks of data. So for example, if a stream of data contains an
+-- empty @ByteString@, it is still treated as empty, and the consuming
+-- function is not called.
+--
+-- @since 1.0.6
+peekForeverE :: (Monad m, MonoFoldable i)
+             => ConduitM i o m ()
+             -> ConduitM i o m ()
+peekForeverE inner =
+    loop
+  where
+    loop = do
+        mx <- peekE
+        case mx of
+            Nothing -> return ()
+            Just _ -> inner >> loop
